
package WebFlow.xml;
import org.w3c.dom.*;
import org.xml.sax.*;

import java.io.*;
import java.net.*;
import java.util.*;
import javax.servlet.*;
import java.lang.*;
import WebFlow.event.*;
import WebFlow.*;

public class WebflowManager extends GenericServlet
{
  public Element configFile;
  boolean isMaster;
  org.omg.CORBA.ORB orb;
  public int counter ;
  public void init(ServletConfig Params) throws ServletException
  {
    String[] args = {""};
    orb = org.omg.CORBA.ORB.init(args, new java.util.Properties());
    counter = 0;
  }

  public void service(ServletRequest req, ServletResponse res)
       throws ServletException, IOException
  {

    System.out.println("service:..:"+getServletConfig());
   
    String scheme = req.getScheme();
    String host = req.getServerName();
    String port = Integer.toString(req.getServerPort());
       
    String uri = req.getParameter("installer");
    String installerURL = scheme+"://"+host+":"+port+
                          "/servlets/wfm?installer="+uri; 
    System.out.println("service:uri:"+uri+" installerURL:"+installerURL);
   
    if (uri !=null){
      System.out.println("xmlConfigFile:"+uri);
      //get config file
      getConfigFile(uri);
      //get server IOR
      String serverIOR = getServerIOR(uri,installerURL);
      //sent it to client
      res.setContentType("text/plain");
      OutputStream os = res.getOutputStream();
      PrintStream out = new PrintStream(os);
      out.println(serverIOR);
    }
  }

  public String makeXMLdoc(String uri, String ID)
  {
    int index = uri.lastIndexOf("/")+1;
    String webflowDTD = uri.substring(1,index)+"webflow.dtd";

    String buf = "<?xml version=\"1.0\"?>\n";
    buf += "<!DOCTYPE WebFlowContext  SYSTEM \""+webflowDTD+"\"\n";
    buf += "[<!ENTITY  timer-master-decls SYSTEM "+uri+">]>\n";
    buf += "<WebFlowContext entityFlag=\"no\" ";
    if (ID != null)
      buf += " componentID=\""+ID+"\"";
    buf += "> &timer-master-decls;\n";
    buf += "</WebFlowContext>";
    System.out.println("makeXMLdoc:buf:"+buf);

    return buf;
  }

  public void  getConfigFile(String uri)
  {
    String buf = makeXMLdoc(uri, null);
    byte[] bar = buf.getBytes();

    Document  root=null;
    Document document=null;
    try{
      DOMParser parser = new DOMParser();
      root = parser.parse(bar);
    }catch (Exception e) {
      e.printStackTrace(System.err);
      return;
    }

    //get dummy WebFlowContext element
    NodeList elements = root.getElementsByTagName("WebFlowContext");
    Element outerContext = (Element)elements.item(0);
 
    //get configFile element
    elements = outerContext.getElementsByTagName("configFile");
    configFile = (Element)elements.item(0);
 
    //get master or slave element
    isMaster = true;
    elements = configFile.getElementsByTagName("master");
    if (elements.getLength() == 0){
      elements = configFile.getElementsByTagName("slave");
      isMaster = false;
    }
    configFile = (Element)elements.item(0);
    System.out.println("getConfigFile: returned:"+configFile);
  }

  public void startServer(String uri, String installerURL )
  {
    System.out.println("startServer:started");
    uri = makeXMLdoc(uri, configFile.getAttribute("componentID"));

    try{
      //create a temporary file
      String suffix = Integer.toString(counter);
      File tf = File.createTempFile("temp",suffix);
      String fileName = tf.getPath();
   
      FileOutputStream fos = new FileOutputStream(tf);
      fos.write(uri.getBytes());

      String display = "mickey.npac.syr.edu:0.0";
      String command = "java WebFlow.test.Server "+fileName+" "+installerURL;
      command = "xterm -sb -d "+display+" -title server_"+(counter++)+
         " -e "+command;
      counter++;
      System.out.println("Command:"+command);
      
      Process  p = Runtime.getRuntime().exec(command);
      printProcessOut ppo = new printProcessOut(p);
      ppo.startProcess();
     
      System.out.println("WebFlow server started..");
    }  
    catch (Exception ex){
      System.out.println("Error in starting ModuleFactory in Proxy:"+ex);
    }
    System.out.println("startServer:returned");
  }

  public String getServerIOR(String uri, String installerURL)
  {
    System.out.println("getServerIOR:started");

    String ior = null;
    //It is master server. If it is down, start it.Then return its ior
    if (isMaster){
      System.out.println("getServerIOR:It is master");
      String localFile  = configFile.getAttribute("localFile");
      System.out.println("getServerIOR:localFile:"+localFile);
      ior = getMasterIOR(false,localFile);
      if (ior == null){
	startServer(uri, installerURL);
	ior = getMasterIOR(true,localFile);
      }
    }
    else{
      System.out.println("getServerIOR:It is slave"); 
      //get slave server by invoking getContext() through master
      //write code here
      String masterURL =  configFile.getAttribute("masterURL");
      String parent    = configFile.getAttribute("parent");
      String componentID = configFile.getAttribute("componentID");
      String objectID = parent+"/"+componentID;
      System.out.println("SLAVE:parent:"+parent+" componentID:"+componentID+
			 " objectID:"+objectID);

      String masterIOR = getMasterIOR(true,masterURL);
      ior = getSlaveIOR(false, masterIOR, objectID);
      if (ior == null){
        startServer(uri, installerURL);
        ior = getSlaveIOR(true, masterIOR, objectID);
      }
    }
     System.out.println("getServerIOR:returned");
    return ior;
  }

  public String getSlaveIOR(boolean isWaiting,String masterIOR,
			    String objectID)
  {
    System.out.println("getSlaveIOR ENTERED..:"+objectID);

    org.omg.CORBA.Object master = orb.string_to_object(masterIOR);
    WebFlowContext masterRef = WebFlowContextHelper.narrow(master);
    org.omg.CORBA.Object slaveRef=null;
    boolean flag=true;
    while(flag){
      try{
	slaveRef = masterRef.getContext(objectID);
	//System.out.println("getSlaveIOR:slaveRef:"+slaveRef);
      }catch(java.lang.NullPointerException ex){
	//System.out.println("getSlaveIOR got exeption :"+ex);
      }
      
      boolean non_exist = true;
      try{
	//non_exist = slaveRef._non_existent();
	non_exist = WebFlow.ProxyImpl.nonExistent(slaveRef);
      }catch(org.omg.CORBA.COMM_FAILURE ex){slaveRef=null;}
       catch(org.omg.CORBA.OBJECT_NOT_EXIST ex){slaveRef=null;}
       catch(org.omg.CORBA.INV_OBJREF ex){slaveRef=null;}
       catch(java.lang.NullPointerException ex){
	 slaveRef=null;//System.out.println("getSlaveIOR:EXC :"+ex);
       }
       //Don't wait any more. Just exit
       if (!isWaiting){
	 if (non_exist)
	   slaveRef= null;
	 break;

       }

      if (slaveRef != null)
	flag = false;
      try{
	Thread.sleep(500);
      }catch(InterruptedException ex){}
      
    }
    System.out.println("getSlaveIOR:after loop:"+slaveRef);
    String slave=null;
    if (slaveRef != null){
      System.out.println("getSlaveIOR:slaveRef is not NULL");
      slave = orb.object_to_string(slaveRef);
    }
    
    return slave;
  }
  
  public String  getMasterIOR(boolean isWaiting,String uri)
  {
    org.omg.CORBA.Object obj = null;
    String ref=null;

    boolean flag=true;
    while(flag){
      ref = readIOR(uri);
      obj = orb.string_to_object(ref);
      boolean non_exist = true;
      try{
	//non_exist = obj._non_existent();
	non_exist = WebFlow.ProxyImpl.nonExistent(obj);
      }catch(org.omg.CORBA.COMM_FAILURE ex){ref=null;}
      catch(org.omg.CORBA.OBJECT_NOT_EXIST ex){ref=null;}
      catch(org.omg.CORBA.INV_OBJREF ex){ref=null;}
      catch(java.lang.NullPointerException ex){
	System.out.println("getMasterIOR :"+ex);ref=null;}
      
      if (!isWaiting){
	if (non_exist)
	  ref = null;
	break;

      }
      if (!non_exist)
	flag = false;
      try{
	Thread.sleep(500);
      }catch(InterruptedException ex){
      }
    }
    return ref;
  }


public String readIOR(String uri)
  {
    if (uri.indexOf("http://") < 0)
      return readIORFromFile(uri);
    else
      return readIORFromURL(uri);
  }

  public  String readIORFromFile(String fn)
{
  //System.out.println("readIORFromFile:entered");
    String ref = null;
    try
      {
	String refFile = fn;//"Hello.ref";
	FileInputStream file = new FileInputStream(refFile);
	BufferedReader in =
	  new BufferedReader(new InputStreamReader(file));
	ref = in.readLine();
	file.close();
      }
    catch(IOException ex)
      {
	System.err.println("Can't read from `" +
			   ex.getMessage() + "'");
	return null;
	//System.exit(1);
      }
    //System.out.println("readIORFromFile:returned:"+ref);
    return ref;
        
  }

  public String  readIORFromURL(String url)
  {
  System.out.println("readIORFromURL:entered:"+url);
  String masterServerIOR="";
  String thisLine=null;
  try {
    URL u = new URL(url);
    // now turn the URL into a DataInputStream
    DataInputStream theHTML = new DataInputStream(u.openStream());
    while ((thisLine = theHTML.readLine()) != null) 
      masterServerIOR += thisLine;
  } catch (Exception e) {
    System.out.println("configReader:getIORfromURL:"+e);
  }
  System.out.println("readIORFromURL:returned:"+masterServerIOR);
  return masterServerIOR;
  }

}
