

package WebFlow;
import WebFlow.event.*;
import java.util.*;

/**
 * This class manages hookups between properties, so that a
 * bound property change on object X turns into a property
 * set on a related property on object Y.
 * <P>
 * We do this by associating a PropertyHookup adaptor with each
 * source object that we are interested in.  As part of the adaptor
 * we keep track of which target   setter methods to call when a given
 * property changes.
 */

import java.lang.reflect.*;
import java.io.*;
import java.util.Hashtable;
import java.util.Vector;
import org.omg.CORBA.*;
import org.omg.CORBA.InterfaceDefPackage.*;

public class WebFlowContextOps extends BeanContextSupport 
				   implements WebFlowContextOperations
{   
    String ContextName;
    private moduleFactory  MFImpl,factory;
    private configReader   configSettings_;
    private org.omg.CORBA.ORB orb_;

    // Table that maps from property names to a vector of HookupTargets
    //sourceBinding from source to target
    //targetBinding from target to source
    Hashtable sourceBinding, targetBinding;
    int myColor;
    ServerRequest request;


    /**
     *The constructor for implementation of WebFlowContext interface. This is
     *used internally  for only for instantiating 
     *outermost WebFlowContext container objects.
     *@param peer org.omg.CORBA.Object server object using 
     * this implementation for processing  user requests
     * @configSettings configReader configuration server
     * @orb ORB  ORB CORBa object
     * @ContextName String the relative context name 
     * @throws NullPointerException <tt>ContextName</tt> is Null.
     */
    public WebFlowContextOps(org.omg.CORBA.Object peer,
				 configReader configSettings, ORB orb,
			        String ContextName) 
				  throws NullPointerException
    {
	super(peer, orb);	
	//WebFlowContextPeer = peer;	
      
	System.out.println("WebFlowContextOps :Outermost Context is being created:"+ContextName);

	//store the webFlow server name
	this.ContextName = ContextName;
	
	sourceBinding = new Hashtable();
	targetBinding = new Hashtable();
	orb_  = orb;
	configSettings_ = configSettings;

	//Add myself to WebFlow server pool
	configSettings_.addWFServer(ContextName, peer); 

    }

  /**
   *The constructor for implementation of WebFlowContext interface. This is
   *used for only for instantiating inner WebFlowContext container objects.
   * @param peer org.omg.CORBA.Object server object using 
   *  this implementation for processing  user requests
   * @configSettings configReader configuration server
   * @orb ORB  ORB CORBa object
   * @ContextName String the relative context name 
   * @factory moduleFactory The Object factory reference
   * @throws NullPointerException <tt>ContextName</tt> is Null.
   */
    public WebFlowContextOps(org.omg.CORBA.Object peer,
		 configReader configSettings, ORB orb,
		 String ContextName, moduleFactory factory)
		  throws NullPointerException
    {
	super(peer, orb);	
	WebFlowContextPeer = peer;	
      
	//store the webFlow server name
	this.ContextName = ContextName;
      
	sourceBinding = new Hashtable();
	targetBinding = new Hashtable();
	orb_  = orb;
	configSettings_ = configSettings;

	System.out.println("WebFlowContextOps Second:name:"+ContextName+" config:"+configSettings_);
	//Add myself to WebFlow server pool
	configSettings_.addWFServer(ContextName, peer); 
    
	this.factory = factory;
	this.myColor = factory.getColor();
	MFImpl       = factory;
	System.out.println("Context:"+ ContextName+" 's Color :"+myColor);
    }

 public void saveObjectInDataBase()
  {
    String OID = getObjectID();
    
    int lastSlash = OID.lastIndexOf("/");
    if (lastSlash < 0)
      lastSlash = OID.length();
    String parentID = OID.substring(0, lastSlash);
    
    if (OID.equals(parentID))
      parentID = null;

    String objectIOR = orb_.object_to_string(getMyProxy());
    configSettings_.saveObjectInDataBase(OID,parentID,objectIOR,"C");
  }

  public void saveObjectInDataBase( org.omg.CORBA.Object childNode,
				    String childID)
  {
    String OID = childID;
    String parentID = getObjectID();
    String objectIOR = orb_.object_to_string(getMyProxy());
    configSettings_.saveObjectInDataBase(OID,parentID,objectIOR,"M");
  }
 /**
   * save the state of my children, including contexts,user modules and
   * their attributes and return the result as a string
   **/
public void saveStateInDataBase()
{
  //System.out.println("WebFlowContextOps saveStateInXml:"+getObjectID());
  //String relativeName=ContextName.substring(ContextName.lastIndexOf("/")+1);
  String relativeName = ContextName;
  String  xmlFragment= "<WebFlowContext componentID=\""+relativeName+"\">\n";
  saveObjectInDataBase();

  //This is Depth First Search (DFS) strategy
  org.omg.CORBA.Object childNode = null;
  myHashMap targets = (myHashMap) children.clone();	  
  java.util.Iterator iter = targets.entrySet().iterator();
  while (iter.hasNext()) {
    Map.Entry entry  = (Map.Entry)iter.next();
    String childID   = (String)entry.getKey();//Child Object
    BCSChild bcsChild= (BCSChild)entry.getValue(); //BCSChildchild value
    childNode        = (org.omg.CORBA.Object)bcsChild.getChild(); 
    
    if (childNode._is_a("IDL:WebFlow/BeanContext:1.0")){
      System.out.println("WebFlowContextOps Context:childID:"+
			 childID+" obj:"+childNode);
      WebFlowContext wfc = WebFlowContextHelper.narrow(childNode);
      System.out.println("WebFlowContextOps wfc obj:"+wfc);
      wfc.saveStateInDataBase();
    }
    else if (childNode._is_a("IDL:WebFlow/BeanContextChild:1.0")){
      //save properties in DataBase table
      BeanContextChild bcc = BeanContextChildHelper.narrow(childNode);
      bcc.savePropertiesWithJDBC("attributes_table");
      //save reference to object itself
      saveObjectInDataBase(childNode, childID);
    }
  }
  //now save the binding table of this context
  iter = targets.entrySet().iterator();
  while (iter.hasNext()) {
    Map.Entry entry  = (Map.Entry)iter.next();
    String childID   = (String)entry.getKey();//Child Object
    saveConnectionTableInDataBase(childID);
  }

}

  /**
   * save the state of my children, including contexts,user modules and
   * their attributes and return the result as a string
   **/
public String saveStateInXml(String saveCase)
{
  //System.out.println("WebFlowContextOps saveStateInXml:"+getObjectID());
  //String relativeName=ContextName.substring(ContextName.lastIndexOf("/")+1);
  String relativeName = ContextName;
  String  xmlFragment= "<WebFlowContext componentID=\""+relativeName+"\">\n";

  //This is Depth First Search (DFS) strategy
  org.omg.CORBA.Object childNode = null;
  myHashMap targets = (myHashMap) children.clone();	  
  java.util.Iterator iter = targets.entrySet().iterator();
  while (iter.hasNext()) {
    Map.Entry entry  = (Map.Entry)iter.next();
    String childID   = (String)entry.getKey();//Child Object
    BCSChild bcsChild= (BCSChild)entry.getValue(); //BCSChildchild value
    childNode        = (org.omg.CORBA.Object)bcsChild.getChild(); 
    
    if (childNode._is_a("IDL:WebFlow/BeanContext:1.0")){
      System.out.println("WebFlowContextOps Context:childID:"+
			 childID+" obj:"+childNode);
      WebFlowContext wfc = WebFlowContextHelper.narrow(childNode);
      System.out.println("WebFlowContextOps wfc obj:"+wfc);
      xmlFragment +=  wfc.saveStateInXml(saveCase);
    }
    else if (childNode._is_a("IDL:WebFlow/BeanContextChild:1.0")){
      System.out.println("WebFlowContextOps Child:childID:"+childID);
      String componentRef = childID.substring(childID.lastIndexOf("/")+1);
      xmlFragment += "<ModuleInstance>\n";
      xmlFragment += "<componentRef>"+componentRef +"</componentRef>\n";
      xmlFragment += "<moduleID>"+childID +"</moduleID>\n";

      BeanContextChild bcc = BeanContextChildHelper.narrow(childNode);
      xmlFragment +=  bcc.saveXMLProperties(saveCase);
      //xmlFragment +=  configSettings_.saveXMLProperties(childNode, 
      //					    componentRef, saveCase);
      xmlFragment += "</ModuleInstance>\n";
    }
  }
  //now save the binding table of this context
  iter = targets.entrySet().iterator();
  while (iter.hasNext()) {
    Map.Entry entry  = (Map.Entry)iter.next();
    String childID   = (String)entry.getKey();//Child Object
    xmlFragment += "<connections >\n";
    xmlFragment += saveConnectionTable(childID);
    xmlFragment += "</connections >\n";
  }
  xmlFragment += "</WebFlowContext>\n";
  return xmlFragment;

}

//
//save the state of connections
//
public String  saveConnectionTable(String sourceModuleObjectID )
{
  String xmlFragment="";
  Vector bindings = getBindings(sourceModuleObjectID, sourceBinding);
  //remove connection entries belonging to this removed child
  for(int k=0;k<bindings.size();k++){
    HookupTarget elem = (HookupTarget)bindings.elementAt(k);
    String sourceObjectID  = elem.getSourceObjectID();
    String targetObjectID  = elem.getTargetObjectID();
    String targetMethod  = elem.getTargetMethod();
    String eventRef  = elem.getEventID();
    String typeOfConnection="";
    int flowModel = elem.getFlowModel();
    if (flowModel==HookupTarget.PUSH)
      typeOfConnection = "push";
    else if (flowModel ==HookupTarget.PULL)
      typeOfConnection = "pull";

    xmlFragment += "<connect eventRef=\""+eventRef+
                     "\" typeOfConnection=\""+typeOfConnection+"\">\n";
    xmlFragment += "<sourceObject>\n";
    xmlFragment += "<moduleRef>"+sourceObjectID+"</moduleRef>\n";
    xmlFragment += "</sourceObject>\n";
    xmlFragment += "<targetObject>\n";
    xmlFragment += "<moduleRef>"+targetObjectID+"</moduleRef>\n";
    xmlFragment += "<targetMethod>"+targetMethod+"</targetMethod>\n";
    xmlFragment += "</targetObject>\n";
    xmlFragment += "</connect>\n";
  }
  return xmlFragment;
}

//
//save the state of connections in DataBase
//
public void   saveConnectionTableInDataBase(String sourceModuleObjectID )
{
  String xmlFragment="";
  Vector bindings = getBindings(sourceModuleObjectID, sourceBinding);
  //remove connection entries belonging to this removed child
  for(int k=0;k<bindings.size();k++){
    HookupTarget elem = (HookupTarget)bindings.elementAt(k);
    String sourceObjectID  = elem.getSourceObjectID();
    String targetObjectID  = elem.getTargetObjectID();
    String targetMethod  = elem.getTargetMethod();
    String eventRef  = elem.getEventID();
    String typeOfConnection="";
    int flowModel = elem.getFlowModel();
    if (flowModel==HookupTarget.PUSH)
      typeOfConnection = "push";
    else if (flowModel ==HookupTarget.PULL)
      typeOfConnection = "pull";

    configSettings_.saveConnectionTableInDataBase(sourceObjectID,eventRef,
			    targetObjectID,targetMethod,typeOfConnection);
  }
}


/**
 * This method is called internally by the system when user wants to clean up
 * the object of this implemenation. This includes removing IR and 
 * module factory created internally.
 */

public void deactivate()
{
    System.out.println("WebFlowContextOps(deactivate) entered..");

    try{
      WebFlowContextImpl impl = (WebFlowContextImpl)WebFlowContextPeer;
      Process irep = impl.repository;
      irep.destroy();

      try{
	irep.waitFor();
      }catch(InterruptedException ex){} 

      moduleFactory fac = impl.RedMFImpl;
      fac.deactivate();

    }catch(org.omg.CORBA.COMM_FAILURE ex){
        System.out.println("WebFlowContextOps(deactivate) Exc.:"+ex);
     }

    try{
      WebFlowContextImpl impl = (WebFlowContextImpl)WebFlowContextPeer;
      BOA boa = impl.boa_;
      boa.deactivate_impl(null);
    }catch(org.omg.CORBA.COMM_FAILURE ex){
        System.out.println("WebFlowContextOps(deactivate) Exc.:"+ex);
     }
     System.out.println("WebFlowContextOps(deactivate) finished..");
}   

  /**
   * This method is used internally for disconnecting the context 
   * object from ORB
  **/
public void destroy()
        {
            System.out.println("WebFlowContextOps:"+getObjectID()+
                          "--WAS destroy called..");

	    try{
	      orb_.disconnect(WebFlowContextPeer);
	      WebFlowContextImpl wfimpl=(WebFlowContextImpl)WebFlowContextPeer;
	      //System.out.println("WebFlowContextOps: This CONTEXT RIGTH");
	      orb_.disconnect(wfimpl.WebFlowContextPeer_tie);
	    } catch(org.omg.CORBA.COMM_FAILURE e){
	       System.out.println("WebFlowContextOps:Destroy:exc.:"+e);
	    }
        }      

  /**
   * Find the context for specified name and return reference to it
   * It looks for the  contexts under this context,including this
   * @bindingName String  the context name like this:A/B/C  
   **/

public org.omg.CORBA.Object getContext(String bindingName)
  {
    return (getContextExtended(bindingName, false));
  }

public org.omg.CORBA.Object getParentContext(String bindingName)
  {
    return (getContextExtended(bindingName, true));
  }

public org.omg.CORBA.Object getContextExtended(String bindingName, boolean parent)
  {
    String ID = getObjectID();
    System.out.println("WebFlowContex:getContext:ID:"+ID+" bindingName:"+bindingName);
 
    if (ID.equals(bindingName)){
      System.out.println("WebFlowContex:getContext:FOUND equal");
      return getMyProxy();
    }

    int indx_ = bindingName.indexOf("/", ID.length()+1);
    if (indx_ <0)
      indx_ = bindingName.length();

    String childID = bindingName.substring(0,indx_);
    org.omg.CORBA.Object nextChild = getKeyedChild(childID);
    if (nextChild == null){
      if (parent && myPrefixEqual(bindingName)){
	System.out.println("WebFlowContex:getContext:Send my proxy..");
	return getMyProxy();
      }
      else{
	System.out.println("WebFlowContex:getContext:Sorry send NULL");
	return null;
      }
    }

    BeanContextChild bcc = BeanContextChildHelper.narrow(nextChild);
    WebFlowContext wfc = WebFlowContextHelper.narrow(bcc.getMyProxy());
    return (wfc.getContextExtended(bindingName, parent));
  }

public void setServerRequest(ServerRequest request)
    {
	this.request = request;
    }   
  
  /**
   * Extracting object identifier(object ID) from 
   * the actual initiator of this request "request". This is internal method.
   * @request ServerRequest CORBA ServerRequest object
   **/
public String  extractObjectID(ServerRequest request)
    {
	String ObjectID=null;

	Context context = request.ctx();
	String contextName = context.context_name();
	//System.out.println("WebFlowContextOps:extractObjectID:Context name :"+contextName);
	org.omg.CORBA.NVList nvl = context.get_values("",0, "*");
	//	org.omg.CORBA.CTX_RESTRICT_SCOPE.value, "IRid"); 

	//System.out.println("WebFlowContextOps:extractObjectID:Context name:count:"+nvl.count());	
       	for(int k=0;k<nvl.count();k++){
	    org.omg.CORBA.NamedValue nval=null;
	    try{
		nval = nvl.item(k);
	    }catch(org.omg.CORBA.Bounds e){
		System.out.println("Got exception in extract Context:extractObjectID:"+e);
	    }
	    String name = nval.name();
	    Any    any  = nval.value();
	    String attval = any.extract_string();
	    //System.out.println("WebFlowContextOps:extractObjectID:Contex Info:"+k+" name:"+name+
	    //	       " value:"+attval);
	    if (name.equals("ObjectID")){
		//System.out.println("WebFlowContextOps:extractObjectID:Got Object ID :");
		ObjectID = attval;
	    }
	}
	return ObjectID;
	
    }


public int getMyColor()
  {
    return myColor;
   }

  
   public org.omg.CORBA.Object getSource(org.omg.CORBA.Object src)
  {
    org.omg.CORBA.ORB orb = orb_;//org.omg.CORBA.ORB.init();
    org.omg.CORBA.Request request = src._request("getSource");
    request.set_return_type(orb.create_interface_tc
		      ("IDL:omg.org/CORBA/Object:1.0","Object"));
    request.invoke();
    java.lang.Exception e = request.env().exception();
    if(e != null)
      System.out.println("I got exception : in getSource req. in Binder"+e);
    org.omg.CORBA.Object esrc = request.result().value().extract_Object();
    return esrc;
       
  }

   public void invokeSetterMethod(org.omg.CORBA.Object target,
				 String setter,
				 org.omg.CORBA.Object evt)
  {
    
    org.omg.CORBA.Request request = target._request(setter);
    request.add_in_arg().insert_Object(evt);
    request.invoke();
    java.lang.Exception e = request.env().exception();
    if(e != null)
      System.out.println("PROP:I got exception during :invokeSetterMethod: "+
			 setter+" :"+e);
    
  }


    /**
     * Create a property hookup, so that a change to the named bound
     * property on the source object turns into a call on the "setter"
     * method of the given target object.
     */

  public void invokeSetterMethod(org.omg.CORBA.Object target,
				 String setter,
				 org.omg.CORBA.Any newValue)
                throws PropertyVetoException
  {
    org.omg.CORBA.Request request;
    
    //System.out.println("VECS: invokeSetterMethod CAlled");
    
    Any dummy = orb_.create_any();
    dummy.type(orb_.get_primitive_tc(org.omg.CORBA.TCKind.tk_void));
    NamedValue result = orb_.create_named_value("",dummy,
                                               org.omg.CORBA.ARG_OUT.value);

    org.omg.CORBA.NVList arg_list = orb_.create_list(0);
    arg_list.add_value("propertyEvent",newValue,org.omg.CORBA.ARG_IN.value);
    request  = target._create_request(orb_.get_default_context(),setter,
			       arg_list,result);
    request.exceptions().add(WebFlow.event.PropertyVetoExceptionHelper.type());
      
    request.invoke();
    java.lang.Exception _ob_ex = request.env().exception();
    if(_ob_ex != null)
      {
	org.omg.CORBA.UnknownUserException _ob_uex;
	try
	  {
	    _ob_uex = (org.omg.CORBA.UnknownUserException)_ob_ex;
	  }
	catch(ClassCastException _ob_dummy)
	  {
	    throw (org.omg.CORBA.SystemException)_ob_ex;
	  }
	System.out.println("VECS:I got exception :invokeSetterMethod: "+
		       setter+" :"+_ob_ex);
	try
	  {
	    throw WebFlow.event.PropertyVetoExceptionHelper.extract(_ob_uex.except);
	  }
	catch(org.omg.CORBA.BAD_OPERATION _ob_dummy)
	  {
	  }

	throw new org.omg.CORBA.UNKNOWN();
      }
  }

   public  org.omg.CORBA.Any invokeGetterMethod(org.omg.CORBA.Object source,
				 String getter)
  {
    
    org.omg.CORBA.Request request = source._request(getter);
    request.set_return_type(orb_.get_primitive_tc(
			    org.omg.CORBA.TCKind.tk_any));
    
    request.invoke();
    java.lang.Exception e = request.env().exception();
    if(e != null)
      System.out.println("I got exception during :invokeGettterMethod"+e);
    Any any = request.result().value().extract_any();
    return any;
        
  }
  
 public  void invokeEventListener(org.omg.CORBA.Object source,
				 String getter)
  {
    org.omg.CORBA.Request request = source._request(getter);
    request.add_in_arg().insert_Object(WebFlowContextPeer); //this);
    request.invoke();
    java.lang.Exception e = request.env().exception();
    if(e != null)
      System.out.println("I got exception during :invokeEventListener:method:"
			 +getter+" : "+e);
           
  }

  public  void invokePropertyListener(org.omg.CORBA.Object source,
				 String propertyAdder)
  {
    invokeEventListener(source, propertyAdder);
    
  }

  /**
   * Get Event Object coming with the request "request" to this Context Server
   * This is used internally
   * @request  ServerRequest CORBA request object
   **/
public org.omg.CORBA.Object 
       getEventObject(org.omg.CORBA.ServerRequest request)
    {
	//Forward event firings from source to targets
	//Usually, each event firing ,s.mousePressed(MouseEvent evt)
	//has one argument, which event object 
	//receive event method call
	org.omg.CORBA.ORB _ob_orb = orb_;
	org.omg.CORBA.NVList _ob_list = _ob_orb.create_list(0);
	org.omg.CORBA.Any _ob_aa0 = _ob_orb.create_any();
	_ob_aa0.type(_ob_orb.create_interface_tc(
			    "IDL:omg.org/CORBA/Object:1.0","Object"));
	_ob_list.add_value("", _ob_aa0, org.omg.CORBA.ARG_IN.value);
	request.params(_ob_list);
	//get event Object
	org.omg.CORBA.Object evt = _ob_aa0.extract_Object();
	return evt;
    }
  
  /**
   * Forward the received event object to target. This method first extract
   * the identity of client, evet source, and event Object ID
   * from "request" CORBA object
   * and find the marching entry in translation table to find the registered
   * targets and forward this event to these  targets
   * @request ServerRequest  CORBA server request object
   **/
public void forwardEvent(org.omg.CORBA.ServerRequest request)
    {
	//get event Object
	org.omg.CORBA.Object evt = getEventObject(request);
       
	//find the corresponding source object with same IRid and method name
	String sourceObjectID = extractObjectID(request);
	//System.out.println("WebFlowContextOps:forwardEvent:sourceObjectID:"+sourceObjectID);

	int size = sourceBinding.size();
	String opname = request.op_name();
	//System.out.println("WebFlowContextOps:forwardEvent:"+ContextName+
	//	   " :table size :"+size+" for:"+opname);

	//sourceIDs ---> eventIDs ---> vector of triples (target, targetObjectID,targetMethod)
	Hashtable events = (Hashtable) sourceBinding.get(sourceObjectID);
	//If there is  no hashtable for connections going out of source
	if (events == null){
	    System.out.println("forwardEvent:No Events  for: "+sourceObjectID);
	    return;
	}
	String eventID = opname;
	Hashtable targets = (Hashtable)events.get(eventID);
	if (targets == null){
	    System.out.println("forwardEvent: no Connections for : "+eventID);
  	    System.out.println("forwardEvent:WebFlowCont.:NO target for this event:"+opname);
	    return;
	}

	for (Enumeration elems = targets.elements();elems.hasMoreElements();) {
	    HookupTarget target = (HookupTarget)elems.nextElement();

	    //If target is attached as pull consumer, store its event
	    if (target.flowModel == HookupTarget.PULL){
		putEvent(evt,target);
		continue;
	    }

	    try {
		invokeSetterMethod(target.getTarget(),
				 target.getTargetMethod(),evt);
	    } catch (Exception ex) {
		System.err.println("Unexpected Property set exception");
		ex.printStackTrace();
	    }
	}
    }
  
  /**
   * pull all events which came to the caller of this method and collect them
   * and return in Object array. User will look at these objects and call the
   * relevant method of each event object to extract needed data. 
   * The identity of user is extracted from request object by getting
   * inserted "Corba context information" in this object
   **/

    public org.omg.CORBA.Object[] pullEvents() 
    {
	
	//targetIDs ---> sourceIDs ---> vector of triples 
	//(source, eventID and targetMethod) 
	String targetObjectID = extractObjectID(request);
	//System.out.println("WebflowContext:pullevents:targetObjectID:"+
	//	   targetObjectID);
	Vector bindings = getBindings(targetObjectID, targetBinding);

	Vector evtList = new Vector();
	for(int k=0;k<bindings.size();k++){
	   HookupTarget elem = (HookupTarget)bindings.elementAt(k);
	   if (elem.flowModel == HookupTarget.PULL)
	       evtList.addAll(elem.getEvents());
	}
    
	int nelem = evtList.size();
	org.omg.CORBA.Object []olist=new org.omg.CORBA.Object[nelem];
	for(int k=0;k<nelem;k++){
	  //System.out.println("WebFlowContextOps:ADDING NEXT EVET :");
	  olist[k] = (org.omg.CORBA.Object)evtList.elementAt(k);
	}
	return olist;
   
    }

  /* This method is used internally
   *
   **/
public void putEvent(org.omg.CORBA.Object evt,HookupTarget sourceHook)
    {

      //System.out.println("WebFlowContextOps:putEvent: Entered..."); 
	 String sourceObjectID = sourceHook.getSourceObjectID();
         String targetObjectID = sourceHook.getTargetObjectID();
	 String eventID        = sourceHook.getEventID();
	 Hashtable subscribers = (Hashtable) targetBinding.get(targetObjectID);

	 //If there is  no hashtable for connections going into target 
	 if (subscribers == null){
	     System.out.println("WebFlowContextOps:putEvent:subscribers(NULL):"+
				targetObjectID);
	     return;
	 }
	 Hashtable connections = (Hashtable)subscribers.get(sourceObjectID);
	 if (connections == null){
	     System.out.println("WebFlowContextOps:putEvent :connections(NULL):"+ 
				sourceObjectID);
	     return;
	 }
	 String index = sourceObjectID+eventID;
	 HookupTarget onetarget = (HookupTarget)connections.get(index);
	 if (onetarget == null){
	     System.out.println("WebFlowContextOps:putEvent:onetarget(NULL):"+index);
	     return;
	 }
	 onetarget.putEvent(evt);
	
    }
   
  /**
   * Attach an property change notification with propertyName,"propertyName"
   * from source object to target object. "targetMethod" method of target
   * is called with the parameter of "PropertyChangeEvent" event whenever
   * property change event is fired from source. User has to create source
   * and target objects with "createNewModule" method of this class before.
   * @source org.omg.CORBA.Object Event source  Object
   * @propertyName String the property name
   * @target  org.omg.CORBA.Object Event target  Object
   * @targetMethod String the name of method which is called whenever
   * event with name of "propertyName" is fired from source object
   **/
public void attachPushProperty(org.omg.CORBA.Object source,String propertyName,
			org.omg.CORBA.Object target, String targetMethod) 
{
    String eventID = "propertyChange/"+propertyName;
    attachEvent(source, eventID, target, targetMethod,HookupTarget.PUSH);
}

 /**
   * Remove established property change event binding by attachPushProperty
   * method before  from source object to target object. 
   * @source org.omg.CORBA.Object Event source  Object
   * @propertyName String the property name
   * @target  org.omg.CORBA.Object Event target  Object
   * @targetMethod String the name of method which is called whenever
   * event with name of "propertyName" is fired from source object
   **/
public void detachPushProperty(org.omg.CORBA.Object source,String propertyName,
			org.omg.CORBA.Object target, String targetMethod) 
{
    String eventID = "propertyChange/"+propertyName;
    detachEvent(source, eventID, target, targetMethod,HookupTarget.PUSH);
}

/**
   * Attach an Vetoable property change notification with 
   * propertyName,"propertyName"
   * from source object to target object. "targetMethod" method of target
   * is called with the parameter of "VetoableChangeEvent" event whenever
   * property change event is fired from source. User has to create source
   * and target objects with "createNewModule" method of this class before.
   * The difference of vetoable property change from property change is that
   * user can veto the requested propty change and raise exception
   * @source org.omg.CORBA.Object Event source  Object
   * @propertyName String the property name
   * @target  org.omg.CORBA.Object Event target  Object
   * @targetMethod String the name of method which is called whenever
   * event with name of "propertyName" is fired from source object
   **/
public void attachPushVetoableProperty(org.omg.CORBA.Object source,String propertyName,
			org.omg.CORBA.Object target, String targetMethod) 
{
    String eventID = "vetoablePropertyChange/"+propertyName;
    attachEvent(source, eventID, target, targetMethod,HookupTarget.PUSH);
}

/**
   * Remove established vetoable property change event binding 
   * by attachPushVetoableProperty before 
   * from source object to target object. 
   * @source org.omg.CORBA.Object Event source  Object
   * @propertyName String the property name
   * @target  org.omg.CORBA.Object Event target  Object
   * @targetMethod String the name of method which is called whenever
   * event with name of "propertyName" is fired from source object
   **/
//Detach Vetoable property changes 
public void detachPushVetoableProperty(org.omg.CORBA.Object source,String propertyName,
			org.omg.CORBA.Object target, String targetMethod) 
{
    String eventID = "vetoablePropertyChange/"+propertyName;
    detachEvent(source, eventID, target, targetMethod,HookupTarget.PUSH);
}

/**
 * Register target object as a listener for an Generic user-defined event 
   * notification from source object .The identifier ID of event is eventID.
   * "targetMethod" method of target
   * is called with the parameter of "GenericEvent" event whenever
   * specified event is fired from source. User has to create source
   * and target objects with "createNewModule" method of this class before.
   * @source org.omg.CORBA.Object Event source  Object
   * @eventID  String the event ID
   * @target  org.omg.CORBA.Object Event target  Object
   * @targetMethod String the name of method which is called whenever
   * event with ID "eventID" is fired from source object
   **/
public void attachPushEvent(org.omg.CORBA.Object source,String eventID,
			org.omg.CORBA.Object target, String targetMethod) 
{
    attachEvent(source, eventID, target, targetMethod,HookupTarget.PUSH);

}

/**
   * Unregister an generic event association from source to target object
   * made by attachPushEvent method call before.
   * The identifier ID of event is eventID.
   * @source org.omg.CORBA.Object Event source  Object
   * @eventID  String the event ID
   * @target  org.omg.CORBA.Object Event target  Object
   * @targetMethod String the name of method which is called whenever
   * event with ID "eventID" is fired from source object
   **/
public void detachPushEvent(org.omg.CORBA.Object source,String eventID,
			org.omg.CORBA.Object target, String targetMethod) 
{
    detachEvent(source, eventID, target, targetMethod,HookupTarget.PUSH);

}  

//////////////////////////////////////
/**
 * Pull type of event is used to take event data from source when target need
 * This is demand-driven association as opposed to event-driven 
 * in "push" type of events. Target method name is not needed because 
 * target is explicitly pulling event from its parent context. This method
 * method attach pull type of property change event notification. 
 * That means whenever
 * event came to parent from any of children, it will be kept until target
 * need it and explicitly pull.
 * @source org.omg.CORBA.Object Event source  Object
 * @propertyName  String the property name
 * @target  org.omg.CORBA.Object Event target  Object
 **/

//Pull Events
//Attach property changes 
public void attachPullProperty(org.omg.CORBA.Object source,String propertyName,
			org.omg.CORBA.Object target)
{
    String eventID = "propertyChange/"+propertyName;
    attachEvent(source, eventID, target, null,HookupTarget.PULL);
}

/**
 * Remove associated property event binding by attachPullProperty 
 * method before
 * @source org.omg.CORBA.Object Event source  Object
 * @propertyName  String the property name
 * @target  org.omg.CORBA.Object Event target  Object
 **/
public void detachPullProperty(org.omg.CORBA.Object source,String propertyName,
			org.omg.CORBA.Object target)
{
    String eventID = "propertyChange/"+propertyName;
    detachEvent(source, eventID, target, null,HookupTarget.PULL);
}

/**
 * Pull type of event is used to take event data from source when target need
 * This is demand-driven association as opposed to event-driven 
 * in "push" type of events. Target method name is not needed because 
 * target is explicitly pulling event from its parent context. This method
 * method attach pull type of vetoable property change event notification. 
 * That means whenever  event came to parent from any of children, 
 * it will be kept until target  need it and explicitly pull.
 * The difference of vetoable property change from property change is that
 * user can veto the requested propty change and raise exception
 * @source org.omg.CORBA.Object Event source  Object
 * @propertyName  String the property name
 * @target  org.omg.CORBA.Object Event target  Object
 **/
public void attachPullVetoableProperty(org.omg.CORBA.Object source,String propertyName,
			org.omg.CORBA.Object target)
{
    String eventID = "vetoablePropertyChange/"+propertyName;
    attachEvent(source, eventID, target, null,HookupTarget.PULL);
}

/**
 * Remove associated vetoable property change event binding 
 * by attachPullVetoableProperty  method before
 * @source org.omg.CORBA.Object Event source  Object
 * @propertyName  String the property name
 * @target  org.omg.CORBA.Object Event target  Object
 **/
public void detachPullVetoableProperty(org.omg.CORBA.Object source,String propertyName,
			org.omg.CORBA.Object target)
{
    String eventID = "vetoablePropertyChange/"+propertyName;
    detachEvent(source, eventID, target, null,HookupTarget.PULL);
}

/**
 * Pull type of event is used to take event data from source when target need
 * This is demand-driven association as opposed to event-driven 
 * in "push" type of events. Target method name is not needed because 
 * target is explicitly pulling event from its parent context. This method
 * method attach pull type of generic user-defined event notification. 
 * That means whenever  event came to parent from any of children, 
 * it will be kept until target  need it and explicitly pull.
 * @source org.omg.CORBA.Object Event source  Object
 * @eventID  String the user-defined event ID
 * @target  org.omg.CORBA.Object Event target  Object
 **/
public void attachPullEvent(org.omg.CORBA.Object source,String eventID,
			org.omg.CORBA.Object target)
{
    attachEvent(source, eventID, target, null,HookupTarget.PULL);

}

/**
 * Remove associated user-defined generic event binding 
 * by attachPullEvent  method before
 * @source org.omg.CORBA.Object Event source  Object
 * @propertyName  String the property name
 * @target  org.omg.CORBA.Object Event target  Object
 **/
public void detachPullEvent(org.omg.CORBA.Object source,String eventID,
			org.omg.CORBA.Object target)
{
    detachEvent(source, eventID, target, null,HookupTarget.PULL);

}

/**
 * This method is used internally to make various types of connections
 * between user modules
 **/

//Attach user events
public void attachEvent(org.omg.CORBA.Object source,String eventID,
			org.omg.CORBA.Object target, String targetMethod,
			int flowModel) 
{
  
  //System.out.println("WebFlowContextOps:attachPushEvent: Entered..."); 
    
    BeanContextChild source_bcc = BeanContextChildHelper.narrow(source);
    BeanContextChild target_bcc = BeanContextChildHelper.narrow(target);
    source = source_bcc.getBeanContextChildPeer();
    //System.out.println("WebFlowContextOps:attachPushEvent: After getBeanContextChildPeer...");

    String sourceObjectID = source_bcc.getObjectID();
    String targetObjectID = target_bcc.getObjectID();
    //System.out.println("WebFlowContextOps:attachPushEvent: After get Ids of modules sid:"+ sourceObjectID+" tid:"+targetObjectID+" eventID:"+eventID);

    //sourceIDs ---> eventIDs ---> vector of triples (target, targetObjectID,targetMethod)
    Hashtable events = (Hashtable) sourceBinding.get(sourceObjectID);
    //If there is  no hashtable for connections going out of source
    if (events == null){
      //System.out.println("WebFlowContextOps:AttachEvent: Creating events for :"+sourceObjectID);
	events = new Hashtable();
	sourceBinding.put(sourceObjectID, events);
    }  
    Hashtable connections = (Hashtable)events.get(eventID);
    if (connections == null){
      //System.out.println("WebFlowContextOps:AttachEvent: Creating Connections for :"+eventID);
	connections = new Hashtable();
	events.put(eventID, connections);
	//Source always send its events to its container. 
	//Therefore, container adds itself to event source just once, no more 
	org.omg.CORBA.Object myProxy = getMyProxy();
	//System.out.println("WebFlowContextOps:AttachEvent: proxy:"+myProxy);
	source_bcc.addPropertyChangeListener(eventID, myProxy); //WebFlowContextPeer);
    }
    String index = targetObjectID+targetMethod;
    HookupTarget onetarget = (HookupTarget)connections.get(index);
    if (onetarget == null){
      //System.out.println("WebFlowContextOps:AttachEvent: Putting binding for :"+index);
	HookupTarget hook = new HookupTarget(source, sourceObjectID, eventID,
					     target,targetObjectID,targetMethod);
	hook.setFlowModel(flowModel);
	connections.put(index, hook);
    }
    else{
      //System.out.println("WebFlowContextOps:AttachEvent: Connection already there for :"+index);
	return;
    }
    //////Add reverse binding
    //targetIDs ---> sourceIDs ---> vector of triples (source, eventID and targetMethod) 
    Hashtable subscribers = (Hashtable) targetBinding.get(targetObjectID);
    //If there is  no hashtable for connections going into target 
    if (subscribers == null){
      //System.out.println("WebFlowContextOps:AttachEvent:Creating subscribers:"+targetObjectID);
	subscribers = new Hashtable();
	targetBinding.put(targetObjectID, subscribers);
    }
    connections = (Hashtable)subscribers.get(sourceObjectID);
    if (connections == null){
      //System.out.println("WebFlowContextOps:AttachEvent: Creating Connections for :"+  sourceObjectID+" fore REverse binding");
	connections = new Hashtable();
	subscribers.put(sourceObjectID, connections);

	////Attach also myself to target as a listener for "BeanContext" attribute changes
	//If event target is not local but inside another remote context
	org.omg.CORBA.Object myProxy = getMyProxy();
	if (!contains(targetObjectID)){
	  //System.out.println("WebFlowContextOps:AttachEvent:attach beancontext-target:"+ targetObjectID+" from:"+ sourceObjectID);
		       
	    target_bcc.addPropertyChangeListener("propertyChange/beanContext", myProxy);
	    target_bcc.addVetoableChangeListener("vetoablePropertyChange/beanContext", myProxy);
	}
    }
    index = sourceObjectID+eventID;
    onetarget = (HookupTarget)connections.get(index);
    if (onetarget == null){
      //System.out.println("WebFlowContextOps:AttachEvent: Putting Reverse binding for :"+index);
	//connections.put(index, new HookupTarget(source,eventID,targetMethod));
	HookupTarget hook = new HookupTarget(source, sourceObjectID, eventID,
					     target,targetObjectID,targetMethod);
	hook.setFlowModel(flowModel);
	connections.put(index, hook);
    }

    //System.out.println("WebFlowContextOps:AttachEvent: RETURNED");
}

/**
 * Remove all of my children nested recursively first and remove itself
 * from its parent context if any.During removing each child, also we remove
 * the connections made from/to this child. If this context is outermost
 * context , e.g. master server or slave server, 
 * then shut down the server which  this context implements.Shut down
 * includes deactivation of the IR , module Factory and myself.
 * this method will catched by proxy which will remove this object
 * and proxy itself
 **/
  
public void removeMyself()
  {
    
    org.omg.CORBA.Object peer,parent;

    System.out.println("WebFlowContextOp:removeMyself: entered"); 
    
    peer = getMyProxy(); 
    parent = getBeanContext();
    WebFlowContext wfc = WebFlowContextHelper.narrow(parent);
        
    //remove itself from parent
    if (wfc != null){
      System.out.println("WebFlowContextOp:removeMyself:  wfc is NOT null");
      wfc.removeContext(peer);
    }
    //remove local children
    else{ 
      System.out.println("WebFlowContextOp:removeMyself:  wfc is  null");
      removeLocalChildren();
      //clean up factory and configuration server
      //cleanUp();
    }
  }

/**
 * This method is used internally to remove various types of connections
 * made before between user modules
 **/
public void detachEvent(org.omg.CORBA.Object source,String eventID,
			org.omg.CORBA.Object target, String targetMethod,
			int flowModel) 
{
    
    //System.out.println("WebFlowContextOps:DetachEvent: Entered..."); 
  //System.out.println("ATTACHEVENT:Input:Source Connections for ID:"+getObjectID()+" IS "+  sourceBinding.toString());

    BeanContextChild source_bcc = BeanContextChildHelper.narrow(source);
    BeanContextChild target_bcc = BeanContextChildHelper.narrow(target);
    source = source_bcc.getBeanContextChildPeer();
    //System.out.println("WebFlowContextOps:DetachEvent: After getBeanContextChildPeer...");

    String sourceObjectID = source_bcc.getObjectID();
    String targetObjectID = target_bcc.getObjectID();
    //System.out.println("WebFlowContextOps:DetachEvent: After get Ids of modules sid:"+   sourceObjectID+" tid:"+targetObjectID+" eventID:"+eventID);

    //sourceIDs ---> eventIDs ---> vector of triples (target, targetObjectID,targetMethod)
    Hashtable events = (Hashtable) sourceBinding.get(sourceObjectID);
    //If there is  no hashtable for connections going out of source
    if (events == null){
      //System.out.println("WebFlowContextOps:DetachEvent: No event for :"+sourceObjectID);
	return;
    }
    Hashtable connections = (Hashtable)events.get(eventID);
    if (connections == null){
      //System.out.println("WebFlowContextOps:DetachEvent: No Connection for :"+eventID);
	return;
    }
    String index = targetObjectID+targetMethod;
    HookupTarget onetarget = (HookupTarget)connections.get(index);
    if (onetarget == null){
      //System.out.println("WebFlowContextOps:DetachEvent:No index:"+index);
	return;
    }
    connections.remove(index);
    //System.out.println("WebFlowContextOps:DetachEvent: Connection REMOVED");

    //detach myself(container) from the event source as a listener 
    //If source doesn't have any connection after this detachPushEvent operation for this type of event
    if (connections.size() == 0){
	events.remove(eventID);
	//System.out.println("WebFlowContextOps:DetachEvent:Event SOurce removed");
	org.omg.CORBA.Object myProxy = getMyProxy();
   	source_bcc.removePropertyChangeListener(eventID, myProxy);
    }
    if (events.size() == 0){
      //System.out.println("WebFlowContextOps:DetachEvent:Event REMOVED");
      	sourceBinding.remove(sourceObjectID);
    }

    //System.out.println("ATTACHEVENT:Result:Source Connections for ID:"+getObjectID()+" IS "+   sourceBinding.toString());


    //////Remove reverse binding
    //targetIDs ---> sourceIDs ---> vector of triples (source, SourceObjectID and  eventID) 
    Hashtable subscribers = (Hashtable) targetBinding.get(targetObjectID);
    //If there is  no hashtable for connections going into target 
    if (subscribers == null){
      //System.out.println("WebFlowContextOps:DetachEvent:subscribers are null for:"+   targetObjectID);
	return;
    }
    connections = (Hashtable)subscribers.get(sourceObjectID);
    if (connections == null){
      //System.out.println("WebFlowContextOps:DetachEvent:connections are null for :"+   sourceObjectID);
	return;
    }
    index = sourceObjectID+eventID;
    onetarget = (HookupTarget)connections.get(index);
    if (onetarget == null){
      //System.out.println("WebFlowContextOps:DetachEvent:onetarget is null for :"+index);
	return;
    }
    connections.remove(index);

    if (connections.size() == 0){
      //System.out.println("WebFlowContextOps:DetachEvent:connections = 0 for :"+sourceObjectID);
	subscribers.remove(sourceObjectID);
	////Detach also myself from source as a listener for "BeanContext" attribute changes
	//If event target is not local but inside another remote context
	org.omg.CORBA.Object myProxy = getMyProxy();
	if (!contains(targetObjectID)){
	    target_bcc.removePropertyChangeListener("propertyChange/beanContext", myProxy);
	    target_bcc.removeVetoableChangeListener("vetoablePropertyChange/beanContext", myProxy);
	}
    }
    if (subscribers.size() == 0){
      //System.out.println("WebFlowContextOps:DetachEvent:subscribers = 0 for :"+targetObjectID);
	targetBinding.remove(targetObjectID);
    }
    /////////////Remove reverse binding
    //System.out.println("WebFlowContextOps:DetachEvent: REVERSE BInding is removed");
}

/**
 * Remove module instantiation from its parent including  connections
 * which comes from/into this module established before.
 * @sourceModule org.omg.CORBA.Object The reference to module 
 * which will be removed
 */
  public void removeModule(org.omg.CORBA.Object sourceModule)
    {
      //System.out.println("WebFlowContextOps:removeModule:entered:"+ContextName);
	removeModuleProxy(sourceModule, true);
    }  

/**
 * This method is used internally to remove module
 */
     //Remove Module
    public void removeModuleProxy(org.omg.CORBA.Object sourceModule,boolean IsProxyRemoved)
    {
      BeanContextChild source_bcc = BeanContextChildHelper.narrow( sourceModule);
      String childID = source_bcc.getObjectID();
      System.out.println("WebFlowContextOps:removeModule:entered:"+ContextName+   " MID:"+childID);
    
	Vector bindings = getAllBindings(sourceModule, sourceBinding);
	//remove connection entries belonging to this removed child
	for(int k=0;k<bindings.size();k++){
	   HookupTarget elem = (HookupTarget)bindings.elementAt(k);
	   org.omg.CORBA.Object source = elem.getSource();
	   org.omg.CORBA.Object target = elem.getTarget();
	   String targetMethod  = elem.getTargetMethod();
	   String eventID  =elem.getEventID(); 
	   //call detach event for each connection table
	   detachPushEvent(source, eventID, target, targetMethod); 
	}
	//if (bindings.size() == 0)
	//  System.out.println("WebFlowContextOps:removeModule: No event for :");

	//remove my connection entries also If I am anly target for connections made to
	//other local modules in my context
	removeConnectionFromTarget(sourceModule);

	//remove my module from my children list 
	remove(sourceModule);//sourceObjectID);
	//System.out.println("WebFlowContextOps:removeModule:remove real module itself");
	if (!IsProxyRemoved)
	    return;

	
	//Finally remove from orb
	try{   
	    //disconnect object itself and its proxy
	    source_bcc.destroy();
	}catch(org.omg.CORBA.COMM_FAILURE ex){
	    System.out.println("WebFlowContextOps:removeModule:got Exc.:"+ex);
	}
	    
	/*
	//Finally remove from orb
	try{

	    if (source_bcc.isWFServer())
	      source_bcc.removeMyself();
	    else{
	      setChildDeleted(childID);
	      //disconnect object itself and its proxy
	      source_bcc.destroy();
	      }
	  }catch(org.omg.CORBA.COMM_FAILURE ex){
	  System.out.println("WebFlowContextOps:removeModule:got Exc.:"+ex);
	  }
	  */
	
	//System.out.println("WebFlowContextOps:removeModule:RETURNED for:ID:");
    }


//Remove connection entry in translation table. 
//This entry has target field
//specified as a parameter to this method
/**
 * This method is called internally
 **/
    public void removeConnectionFromTarget(org.omg.CORBA.Object targetModule)
    {     
	Vector bindings = getAllBindings(targetModule, targetBinding);
	//remove connection entries belonging to this removed child
	for(int k=0;k<bindings.size();k++){
	   HookupTarget elem = (HookupTarget)bindings.elementAt(k);
	   org.omg.CORBA.Object source = elem.getSource();
	   org.omg.CORBA.Object target = elem.getTarget();
	   String targetMethod  = elem.getTargetMethod();
	   String eventID  =elem.getEventID(); 
	   //call detach event for each connection table
	   detachPushEvent(source, eventID, target, targetMethod); 
	}
	//if (bindings.size() == 0)
	//  System.out.println("WebFlowContextOps:removeConnectionFromTarget: No event for :");
	//System.out.println("WebFlowContextOps:removeConnectionFromTarget :Returned");
    }

/**
 * Remove only the nested children recursively along with the connections
 * of them.Keep myself.if any of nested child is server, shut down it also.
 **/
   public void removeLocalChildren()
  { 
        System.out.println("removeLocalChildren:ENTERED:objectID:"+getObjectID());
 
      //If it is a context, then remove first all of its children and the node itself
      //This is Depth First Search (DFS) strategy
      org.omg.CORBA.Object childNode = null;

      myHashMap targets = (myHashMap) children.clone();	  
      java.util.Iterator iter = targets.entrySet().iterator();
      while (iter.hasNext()) {
	  Map.Entry entry     = (Map.Entry)iter.next();
	  String childID      = (String)entry.getKey();//Child Object
	  BCSChild bcsChild   = (BCSChild)entry.getValue(); //BCSChildchild value
	  childNode           = (org.omg.CORBA.Object)bcsChild.getChild(); 
		 
	  BeanContextChild bcc = BeanContextChildHelper.narrow(childNode);
	  //debug
	  System.out.println("removeLocalChildren: This is CHILD's objectId:"+
			     bcc.getObjectID());
	  //debug

	  if (!childNode._is_a("IDL:WebFlow/BeanContext:1.0"))
	      removeModule(childNode);
	  else {
	    //This is Context Child. First delete children of this context
	    WebFlowContext wfc = WebFlowContextHelper.narrow(childNode);
	    
	    //If this child is SERVER , remove it
	    wfc.removeLocalChildren() ;
	    
	    //and remove this child from my context
	    removeModule(childNode);
	  }
      }
      System.out.println("removeLocalChildren:RETURNED:objectID:"+getObjectID());
    
  }

/**
 * Remove context instantiation from its parent including  connections
 * which comes from/into this context established before.
 * If this context is outermost container, it will be shut down in addition to
 * removing myself from my parent container
 * @node org.omg.CORBA.Object The reference to this context 
 * which will be removed
 */
    public void removeContext(org.omg.CORBA.Object node)
    {
      //System.out.println("Source Connections for ID:"+getObjectID()+" IS "+  sourceBinding.toString());
      //System.out.println("Target Connections for ID:"+getObjectID()+" IS "+  targetBinding.toString());

      //System.out.println("removeContext:ENTERED:myID:"+getObjectID());
	//If the node is just a child Module, remove it
	BeanContextChild bcc = BeanContextChildHelper.narrow(node);
	if (!node._is_a("IDL:WebFlow/BeanContext:1.0")){
	  //System.out.println("removeContext: This is CHILD:objectId:"+bcc.getObjectID());
	    removeModule(node);
	    return;
	}else{
	  //System.out.println("removeContext: This is CONTEXT:objectId:"+bcc.getObjectID());
	    WebFlowContext wfc = WebFlowContextHelper.narrow(node);
	    wfc.removeLocalChildren() ;
	}
	removeModule(node);
    }

  
    //Remove connection entry in translation table. This entry has target field
    //specified as a parameter to this method
/**
 * this method is used internally
 */
    public Vector  getAllBindings(org.omg.CORBA.Object target, Hashtable bindingTable)
    {     
	BeanContextChild target_bcc = BeanContextChildHelper.narrow(target);
      	String targetObjectID = target_bcc.getObjectID();
	//System.out.println("WebFlowContextOps:getAllBindings:entered:ID:"+  targetObjectID);
	return (getBindings(targetObjectID, bindingTable));
    }
        

   //Remove connection entry in translation table. This entry has target field
    //specified as a parameter to this method
 /**
 * this method is used internally
 */
    public Vector  getBindings(String targetObjectID, Hashtable bindingTable)
    {     
      //System.out.println("WebFlowContextOps:getBindings:entered:ID:"+
      //		   targetObjectID);

	Vector list = new Vector();
	//remove connection entries belonging to this removed child;
	Hashtable subscribers = (Hashtable) bindingTable.get(targetObjectID);
	if (subscribers == null)
	    return list;

	Enumeration subscribers_keys = subscribers.keys();
	while(subscribers_keys.hasMoreElements())
	 {
	     String sourceObjectID = (String)subscribers_keys.nextElement(); 
	     Hashtable connections = (Hashtable)subscribers.get(sourceObjectID);
	     Enumeration connections_elems = connections.elements();
	     while(connections_elems.hasMoreElements())
	      {
		  HookupTarget elem = (HookupTarget)connections_elems.nextElement();
		  list.add(elem);
	      }
	 }
	return list;
    }


   //Copying bindings of module "source" to module "target"
   //User should issue this method call to the container of module "source" which 
   //has all connections of "source" module. Module "there" must have same idl interface as source
/**
 * This method provides user with copying the in/out bindings of module
 * "sourceModule"  into module "there". User needs to issue this method
 * to the parent of sourceModule because all of outgoing connections from
 * sourceModule are kept in its parent container
 **/
public void copyModuleBinding(org.omg.CORBA.Object sourceModule, org.omg.CORBA.Object there)
    {
      //System.out.println("WebFlowContextOps:copyModuleBinding:entered:"+ContextName);
	BeanContextChild source_bcc = BeanContextChildHelper.narrow(sourceModule);
      	String sourceObjectID = source_bcc.getObjectID();

	Vector bindings = getAllBindings(sourceModule, sourceBinding);
	//remove connection entries belonging to this removed child
	for(int k=0;k<bindings.size();k++){
	   HookupTarget elem = (HookupTarget)bindings.elementAt(k);
	   org.omg.CORBA.Object source = elem.getSource();
	   org.omg.CORBA.Object target = elem.getTarget();
	   String targetMethod  = elem.getTargetMethod();
	   String eventID  =elem.getEventID(); 
	   //call attach event for each connection table
	   //change only source in the Hookup entry
	   attachPushEvent(there, eventID, target, targetMethod); 
	}

	copyConnectionFromTarget(sourceModule, there);

	firePropertyChange("copyBinding", getMyProxy(), there);
    }  
    
/**
 * this method is used internally
 **/
public void copyConnectionFromTarget(org.omg.CORBA.Object sourceModule, 
				     org.omg.CORBA.Object there)
{
    Vector bindings = getAllBindings(sourceModule, targetBinding);
    //remove connection entries belonging to this removed child
    for(int k=0;k<bindings.size();k++){
	HookupTarget elem = (HookupTarget)bindings.elementAt(k);
	org.omg.CORBA.Object source = elem.getSource();
	org.omg.CORBA.Object target = elem.getTarget();
	String targetMethod  = elem.getTargetMethod();
	String eventID = elem.getEventID(); 
	//call attach event for each connection table
	//change only target in the Hookup entry
	attachPushEvent(sourceModule, eventID, there, targetMethod); 
    }

}

/**
 * this method is used internally
 **/
    public void setInterfaceDesc(org.omg.CORBA.Object product_, InterfaceDef nextDef )

  {
    InterfaceDef idef=null;
    FullInterfaceDescription  fullInterfaceDesc=null;

    //System.out.println("ProxyImpl:setInterfaceDesc:product:"+product_);
    
    if (nextDef == null)
      idef = product_._get_interface();
    else
      idef = nextDef;
    
    //System.out.println("setInterfaceDesc:after get_interface:");

    fullInterfaceDesc = idef.describe_interface();  
    //System.out.println("setInterfaceDesc:after describe_interface"+fullInterfaceDesc);
    //System.out.println("setInterfaceDesc:after describe_interface :IRid:"+  fullInterfaceDesc.id);
   
    //idsVec.add(fullInterfaceDesc.id);
    OperationDescription []ods = fullInterfaceDesc.operations;
    for(int k=0;k<ods.length;k++){ 
      OperationDescription kops = ods[k];
      //interfacedef.put(kops.name,kops);
      //System.out.println(attr+"SetInterfaceDesc:I am putting opname:"+kops.name);
    }   
    
    InterfaceDef []idefs = idef.base_interfaces();
    for (int b=0;b<idefs.length;b++){  
      //System.out.println(attr+"SetInterfaceDesc:Recursing Counter b:"+b);
      setInterfaceDesc(product_, idefs[b]);
    }  
  }

/**
 * this method is used internally
 **/
    public  org.omg.CORBA.Object readIOR(org.omg.CORBA.ORB orb ,String fn)
    {
    String ref = null;
    try
      {
	String refFile = fn;//"Hello.ref";
	FileInputStream file = new FileInputStream(refFile);
	BufferedReader in =
	  new BufferedReader(new InputStreamReader(file));
	ref = in.readLine();
	file.close();
      }
    catch(IOException ex)
      {
            System.err.println("Can't read from `" +
                                   ex.getMessage() + "'");
	    System.exit(1);
      }
 
    org.omg.CORBA.Object obj = orb.string_to_object(ref);
    return obj;
        
  }
 
   
    /**
     * This is the method that gets called when a bound property
     * changes on the source object.
     * We map the property name to a list of targets and then
     * call each of the target "setter" methods.
     */

  
    // Access to information for code generation
    // Target data
/**
 * this method is used internally
 **/
    public Hashtable getTargetsByProperty() {
	return sourceBinding;
    }

     
//this method gives CORBA object reference to the one with Object Key =childID
 /**
 * this method is used internally
 **/
 public org.omg.CORBA.Object getKeyedChild(String childID)
  { 
       System.out.println("getKeyedChild:looking fo Child:"+childID);
      String nextChildID=null;
      org.omg.CORBA.Object child = null;
      java.util.Iterator i = children.entrySet().iterator();
      while (i.hasNext()) {
	  Map.Entry entry     = (Map.Entry)i.next();
	  //Bear in mind that the following child info. is IOR of its Corba object
	  //We used our myHashMap class to do this
	  nextChildID  = (String)entry.getKey(); //Child Object	
	  BCSChild childValue = (BCSChild)entry.getValue(); //BCSChildchild value
	  System.out.println("getKeyedChild:looking for:"+nextChildID);
	  if (nextChildID.equals(childID)){
	      child = childValue.getChild();
	      System.out.println("I found the context with name:"+childID);
	      break;
	  }
	}

      /*
	if (child._is_a("IDL:WebFlow/BeanContext:1.0"))
	    System.out.println("getNamedChild: This child IS CONTEXT");
	if (child._is_a("IDL:WebFlow/BeanContextChild:1.0"))
	    System.out.println("getNamedChild: This child IS CHILD");
      */
      System.out.println("I exited loop in get_context for:"+nextChildID+
			 " child :"+child);
      return child;
  }

 //this method gives CORBA object reference to the one with Object Key=childID
 /**
 * this method is used internally
 **/
 public org.omg.CORBA.Object[] getSameKeyedChildren(String childID)
  {   
      Vector vl = new Vector();
      childID = getObjectID()+"/"+childID;

      System.out.println("WebFlowContextOps:getSameKeyedChildren:looking fo Child:"+childID+" .............................");
      String nextChildID=null;
      org.omg.CORBA.Object child = null;
      java.util.Iterator i = children.entrySet().iterator();
      while (i.hasNext()) {
	  Map.Entry entry     = (Map.Entry)i.next();
	  //Bear in mind that the following child info. is IOR of its Corba object
	  //We used our myHashMap class to do this
	  nextChildID  = (String)entry.getKey(); //Child Object	
	  BCSChild childValue = (BCSChild)entry.getValue(); //BCSChildchild value
	  System.out.println("getKeyedChild:looking for:"+nextChildID);
	  if (nextChildID.equals(childID)){
	      child = childValue.getChild();
	      System.out.println("I found the context with name:"+childID);
	      vl.add(child);
	  }
      }
      
      java.lang.Object[] ol = vl.toArray();
      int chlen = ol.length;
      org.omg.CORBA.Object[] colist = new org.omg.CORBA.Object[chlen];
      for(int k =0;k<chlen;k++)
	colist[k] = (org.omg.CORBA.Object)ol[k];

      System.out.println("I exited loop in get_context for:"+
			 childID+".........................");
      return colist;
  }


/**
 * marks the child with ID "childID" as deleted. This is called internally
 * when deleting a child
 **/
public void  setChildDeleted(String childID)
  { 
    System.out.println("setChildDeleled:looking for Child:"+childID);
      String nextChildID=null;
      BCSChild childValue=null;
      org.omg.CORBA.Object child = null;
      java.util.Iterator i = children.entrySet().iterator();
      while (i.hasNext()) {
	  Map.Entry entry     = (Map.Entry)i.next();
	  nextChildID  = (String)entry.getKey(); //Child Object	
	  childValue = (BCSChild)entry.getValue(); //BCSChildchild value
	  if (nextChildID.equals(childID)){
	      child = childValue.getChild();
	      System.out.println("setChildDeleled:found :"+childID);
	      break;
	  }
      }
      if (childValue != null){
	System.out.println("setChildDeleled:Set to Deleted:"+childID);
	childValue.setDeletePending(true);
      }
  }

/**
 * checks whether all of my children is deleted or not. If deleted,
 * returns true , otherwise false
 **/
  public boolean   isAllChildrenDeleted()
  { 
      System.out.println("isAllChildrenDeleted:looking for Child:");
      boolean flag = true;
      String nextChildID=null;
      BCSChild childValue;
      org.omg.CORBA.Object child = null;
      java.util.Iterator i = children.entrySet().iterator();
      while (i.hasNext()) {
	  Map.Entry entry     = (Map.Entry)i.next();
	  nextChildID  = (String)entry.getKey(); //Child Object	
	  childValue = (BCSChild)entry.getValue(); //BCSChildchild value
	  flag = flag && childValue.isDeletePending();
      }
      return flag;
  }
  


/**
 * creates an object ID for new instantiated child with name "prodoctName"
 * This method is called internally.  
 * @productName String the name of new module
 **/
  public String createObjectID(String productName)
  {
    productName = productName.substring(productName.lastIndexOf("/")+1);   
    Date  now  = new Date();  
    //String oid = ContextName+"/"+productName;//+"/"+now.toString();
    String oid = getObjectID()+"/"+productName;//+"/"+now.toString();
    return oid;
  }

  
//////////////////////////

/**
 * instantiate new module with name "productName" and inserts it
 * into its children list
 * @productName String the name of new module
 **/
   public org.omg.CORBA.Object addNewModule(String productName)
  {
  
    System.out.println("WebFlowContextImpl:"+ContextName+ 
		" creates product :"+productName);

    org.omg.CORBA.Object product;
    //create a hierarchy of context names like in LDAP directory servers
    productName = ContextName+"/"+productName;
    String oid = createObjectID(productName);
    product = factory.createRemoteProduct(productName, oid); 
    return product;
  }

/**
 * instantiate new context with name "newContextName" and inserts it
 * into its children list
 * @newContextName String the name of new context
 **/
  public org.omg.CORBA.Object addNewContext(String newContextName)
    throws PropertyVetoException,NullPointerException 
  {



    org.omg.CORBA.Object product;

    String oid = createObjectID(newContextName);
    newContextName = ContextName+"/"+newContextName;
    product = factory.createRemoteContext(newContextName, oid);
    
    System.out.println("WebFlowContextOps:"+ContextName+
		" creates context :"+ newContextName+" gives ID:"+oid+
		       " Color of Context:"+myColor);

    return product;
  }

/**
 * It is used internally to add new module
 */
 public void addModule( org.omg.CORBA.Object module, String objectId)
  {
    System.out.println("WebFlowContextImpl:"+ContextName+
	" is adding Module with ID:"+objectId);
	try{
		 addNamedChild(module, objectId);
	}catch(IllegalArgumentException e){
		System.out.println("Excep. in addModule"+e);
	 }
	 catch(NullPointerException e){
		System.out.println("Excep. in addModule"+e);
	 }
     catch(IllegalStateException e){
		System.out.println("Excep. in addModule"+e);
	 }
     System.out.println("WebFlowContextImpl:"+ContextName+
	" is adding Module with ID:"+objectId+" AFTER ...");
	
  }

 /**
 * It is used internally to add new context
 */
  public void addContext(org.omg.CORBA.Object context, String objectId)
  {
    System.out.println("WebFlowContextImpl:"+ContextName+
	" is adding  context with ID:"+objectId+" Color:"+myColor);

    
	try{
		addNamedChild(context,objectId );
	}catch(IllegalArgumentException e){
		System.out.println("Excep. in addContext"+e);
	 }
	 catch(NullPointerException e){
		System.out.println("Excep. in addContext"+e);
	 }
     catch(IllegalStateException e){
		System.out.println("Excep. in addContext"+e);
	 }
	
  }

/**
 * This method is called whenever any of children fired 
 * the property Change Event. The event object come with the paramter "evt"
 * @evt PropertyChangeEvent Property change event object
 **/

 public void propertyChange(PropertyChangeEvent evt) 
      throws PropertyVetoException
   {

     //System.out.println("WebFlowContextOps:propertyChange entered");

	String propertyName   = evt.getPropertyName();
	String sourceObjectID = extractObjectID(request);
	String eventID        = propertyName;
	String tableIndex     = sourceObjectID+eventID;
	
	//System.out.println("WebFlowOps:propertyChange:sourceID:"+sourceObjectID+" eventId:"+eventID);

	//If WebFlowContext got "propertyChange/beanContext" property change event, 
	//  it will call its special method
	if (propertyName.equals("propertyChange/beanContext")){
	    try{
		bc_propertyChange(evt,sourceObjectID);
	    }catch(IllegalArgumentException e){
		System.out.println("propertyChange in WebFlowContextOps:"+e);
	    }catch(IllegalStateException e){
		System.out.println("propertyChange in WebFlowContextOps:"+e);
	    }
	    return;
	}
    	
	//sourceIDs ---> eventIDs ---> vector of triples (target, targetObjectID,targetMethod)
	Hashtable events = (Hashtable) sourceBinding.get(sourceObjectID);
	//If there is  no hashtable for connections going out of source
	if (events == null){
	    System.out.println("propertyChange:No Events  for: "+sourceObjectID);
	    return;
	}
	Hashtable targets = (Hashtable)events.get(eventID);
	if (targets == null){
	    System.out.println("propertyChange: no Connections for : "+eventID);
  	    System.out.println("WebFlowCont.:NO target for this Property Change:"+propertyName);
	    return;
	}
	org.omg.CORBA.Any  newValue = evt.getNewValue();
	for (Enumeration elems = targets.elements();elems.hasMoreElements();) {
	    HookupTarget target	= (HookupTarget)elems.nextElement();

	     //If target is attached as pull consumer, store its event
	    if (target.flowModel == HookupTarget.PULL){
		putEvent(evt,target);
		continue;
	    }

	    try {
	      invokeSetterMethod(target.getTarget(),
				 target.getTargetMethod(),newValue);
	    } catch (PropertyVetoException ex) {
	 	System.err.println("WebFlowContextOps:propertyChange:I got Veto Exception from target");
		//for Vetoable properties throw incoming exeption to source
		throw ex;
	    } catch (Exception ex) {
		//Here I am catching Veto Exceptions I ftarget doesn't want to accept
		//proposed modification sent by source
	 	System.err.println("WebFlowContextOps:propertyChange:"+
				   "Unexpected Property set exception:"+ex);
		ex.printStackTrace();
	    }
	}
    }
    
/**
 * this method is used internally
 **/
     public void propertyChangeAfterVeto(PropertyChangeEvent evt,
					 String sourceObjectID,String eventID  ) 
      throws PropertyVetoException
   {

     //System.out.println("WebFlowContextOps:propertyChangeAfterVeto entered");
	String tableIndex     = sourceObjectID+eventID;

	//sourceIDs ---> eventIDs ---> vector of triples (target, targetObjectID,targetMethod)
	Hashtable events = (Hashtable) sourceBinding.get(sourceObjectID);
	//If there is  no hashtable for connections going out of source
	if (events == null){
	    System.out.println("propertyChangeAfterVeto: No Events  for: "+sourceObjectID);
	    return;
	}
	Hashtable targets = (Hashtable)events.get(eventID);
	if (targets == null){
	    System.out.println("propertyChangeAfterVeto: no Connections for : "+eventID);
  	    System.out.println("WebFlowCont.:NO target for this Property Change:"+eventID);
	    return;
	}
	org.omg.CORBA.Any  newValue = evt.getNewValue();
	for (Enumeration elems = targets.elements();elems.hasMoreElements();) {
	    HookupTarget target	= (HookupTarget)elems.nextElement();

	     //If target is attached as pull consumer, store its event
	    if (target.flowModel == HookupTarget.PULL){
		putEvent(evt,target);
		continue;
	    }

	    try {
	      invokeSetterMethod(target.getTarget(),target.getTargetMethod(),newValue);
	    } catch (PropertyVetoException ex) {
	 	System.err.println("WFS:I got Veto Exception from target");
		//for Vetoable properties throw incoming exeption to source
		throw ex;
	    } catch (Exception ex) {
	 	System.err.println("Unexpected Property set exception");
		ex.printStackTrace();
	    }
	}
    }

/**
 * This method is called whenever any of children fired 
 * the Vetoable property Change Event. 
 * The event object come with the paramter "evt"
 * @evt PropertyChangeEvent Vetoable Property change event object
 **/
    public void vetoablePropertyChange(PropertyChangeEvent evt)
					throws PropertyVetoException 
    {
	String propertyName   = evt.getPropertyName();
	String sourceObjectID = extractObjectID(request);
	String eventID        = propertyName;
	//System.out.println("WebFlowContextOps:vetoablePropertyChange entered:propertyName:"+ propertyName+"source:"+ sourceObjectID+" eventID:"+eventID);

	//If WebFlowContext got "beanContext" property change event, it will call
	//its special method
	if (propertyName.equals("vetoablePropertyChange/beanContext")){
	    bc_vetoableChange(evt,propertyName,sourceObjectID);
	    return;
	}
	
	//System.out.println("vetoablePropertyChange is Called");
	propertyChangeAfterVeto(evt, sourceObjectID,eventID);
    }

/**
 *This method is used internally to process vetoable property change event 
 */
  public void bc_vetoableChange(PropertyChangeEvent pce,String propertyName, String sourceObjectID)
      throws PropertyVetoException 
  {
        BeanContextChild new_bcc = BeanContextChildHelper.narrow(pce.getNewValue().extract_Object());
	String newObjectID       = (new_bcc==null)?null:new_bcc.getObjectID();
	String myObjectID        = getObjectID();
	//System.out.println("WebFlowContextOps: bc_vetoableChange propertyname:"+  pce.getPropertyName()+" my ObjectId:"+myObjectID );

	synchronized(children) {
	    if ("vetoablePropertyChange/beanContext".equals(propertyName) && (contains(sourceObjectID)) &&
	       (!myObjectID.equals(newObjectID )))
	    {
		if (!validatePendingRemove(sourceObjectID)) {
                    System.out.println("WebFlowContextOps: bc_vetoableChange:throw new");
		    throw new PropertyVetoException("current BeanContext vetoes setBeanContext()",pce);
		}
	      	else{
		    System.out.println("WebFlowContextOps: bc_vetoableChange :setRemovePending");
		    ((BCSChild)children.get(sourceObjectID)).setRemovePending(true);
		}
	    }
	}
    }

 
/**
 *This method is used internally to process property change event 
 */
public void bc_propertyChange(PropertyChangeEvent pce, String sourceObjectID)
	throws IllegalArgumentException,IllegalStateException
   {

      	String propertyName			 = pce.getPropertyName();
	//System.out.println("WebFlowContextOps: bc_propertyChange propertyname:"+propertyName);
	
	synchronized(children) {
	    if ("propertyChange/beanContext".equals(propertyName) &&
	        contains(sourceObjectID)		       &&
	        ((BCSChild)children.get(sourceObjectID)).isRemovePending()) 
		{
		    BeanContextChild old_bcc = BeanContextChildHelper.narrow(
							     pce.getOldValue().extract_Object());
		    BeanContextChild new_bcc = BeanContextChildHelper.narrow(
							     pce.getNewValue().extract_Object());
		    String myObjectID        = getObjectID();
		    String oldObjectID       = old_bcc.getObjectID();
		    String newObjectID =null;
		    if (new_bcc != null)
			newObjectID = new_bcc.getObjectID();

		    //System.out.println("WebFlowContextOps: bc_propertyChange old:"+oldObjectID+" new:"+newObjectID+" for:"+propertyName);
		    if ((myObjectID.equals(oldObjectID)) && (!myObjectID.equals(newObjectID)))
			{
			  //    System.out.println("WebFlowContextOps: bc_propertyChange remove:"+  sourceObjectID);
			    remove(sourceObjectID, false);
			} else {
			    System.out.println("WebFlowContextOps: bc_propertyChange setPending");
			    ((BCSChild)children.get(sourceObjectID)).setRemovePending(false);
			}
		    return;
		}
	}
	
	//If I got  "BeanContext" property change from another child 
	//which is inside remote context, remove connection established from one of my children
	//to the remote child
	if ("propertyChange/beanContext".equals(propertyName) &&
	    (!contains(sourceObjectID))){
	    BeanContextChild new_bcc = BeanContextChildHelper.narrow(
							     pce.getNewValue().extract_Object());
	    //If remote child to which we have connection is removed
	    if (new_bcc == null){
	      //System.out.println("WebFlowContextOps: bc_propertyChange:remove remote child");
		removeConnectionFromTarget(pce.getSource());

	    }
	}
	if ("propertyChange/copyBinding".equals(propertyName) &&
	    (!contains(sourceObjectID))){
	    BeanContextChild target = BeanContextChildHelper.narrow(
							     pce.getOldValue().extract_Object());
	    BeanContextChild there = BeanContextChildHelper.narrow(
							     pce.getNewValue().extract_Object());
	    //If remote child to which we have connection is removed
	    //System.out.println("WebFlowContextOps: bc_propertyChange:remove remote child");
	    copyConnectionFromTarget(target, there);
	}

	
   }

/**
 * This method is called by slave server to insert itself to my children
 * @name String the name of slave server
 * @server org.omg.CORBA.Object the svale server reference
 **/
  public void addWFServer(String name, org.omg.CORBA.Object server)
	{
	    System.out.println("addWFServer: adding name:"+name+" config Server:"+configSettings_);
		configSettings_.addWFServer(name, server);
	}
    
/**
 * This method is called by slave server to remove itself from my children
 * @name String the name of slave server
 **/
    public void removeWFServer(String name)
	{
		configSettings_.removeWFServer(name);
	}

/**
 * This method returns the available slave servers currently available.
 * It returns the names of slave servers. This is deprecated. User should use
 * to getContext method to get reference to one slave server and
 * toArray method to get all of my children
 **/
	public  String[] getAllWFServers()
	{
		return (configSettings_.getAllWFServers());
	}

/**
 * This is used internally
 **/
public org.omg.CORBA.Object getGatewayProxy()
	{
	  String serverName = ContextName;
	  org.omg.CORBA.Object server = configSettings_.getWFServer(serverName);
	  System.out.println("getGatewayProxy :server:"+server+" CN:"+serverName);
	  org.omg.CORBA.Object proxy=null;
		  
	  WebFlowContext wc = WebFlowContextHelper.narrow(server);
	  proxy   = factory.prevChild().createProxy(serverName, server);
	  System.out.println("getWFServer :proxy:"+proxy);
		
	  //set my Proxy
	  setMyProxy(proxy);
		
	  return proxy;
	}	
       
/**
 * This is used internally to disconnect myself from ORB.
 * It is called by my proxy
 **/
  public void cleanUp()
 {
     System.out.println("WebFlowContextOps:CLEANUP: Called...");

     if (getBeanContext() == null)
       return;
     
     moduleFactory redf = factory.prevChild();

     //disconnect Red Factory
     redf.dispose();
     
     //disconnect configuration server
     orb_.disconnect(configSettings_);
    
     //disconnect black factory
     orb_.disconnect(factory);

     System.out.println("WebFlowContextOps:CLEANUP: Factories,config disconnect...");

     //disconnect peer
     orb_.disconnect(WebFlowContextPeer);

     //disconnect tie object
     WebFlowContextImpl impl = (WebFlowContextImpl)WebFlowContextPeer;
     _WebFlowContextImplBase_tie tie = impl.WebFlowContextPeer_tie;
     orb_.disconnect(tie);
     System.out.println("WebFlowContextOps:CLEANUP: Peer and tie disconnect...");

}

/**
 * This is deprecated method. It creates new context with name "userName"
 * inside slave server "serverName". It returns the reference to proxy of
 * new created context. User should use getContext Method instead of this
 **/

   public org.omg.CORBA.Object getWFServer(String serverName, 
					   String userName)
	{
		org.omg.CORBA.Object server = configSettings_.getWFServer(serverName);
		System.out.println("getWFServer :server:"+server);
		org.omg.CORBA.Object proxy=null;
		  
		WebFlowContext wc = WebFlowContextHelper.narrow(server);
		String bcname   = "proxy_user/"+userName;
								
		org.omg.CORBA.Object newContext=null;
		try{
		  //set my Proxy
		  //If server accessed is local
		  if (serverName.equals(ContextName)){
		    newContext = addNewContext(userName);
		    proxy   = factory.prevChild().createProxy(bcname, newContext);
		    WebFlowContextImpl wfci = (WebFlowContextImpl)newContext;
		    wfci.getImpl().setMyProxy(proxy);
		  }
		  //If it is remote
		  else {
		    newContext = wc.addNewContext(userName);
		    proxy   = factory.prevChild().createProxy(bcname, newContext);
		    BeanContextChild bcc = BeanContextChildHelper.narrow(newContext);
		    bcc.setMyProxy(proxy);
		  }

		}catch( PropertyVetoException e){
		}catch(NullPointerException e){}

		BeanContextChild bcc = BeanContextChildHelper.narrow(proxy);
                String oid = bcc.getObjectID();
		 System.out.println("getWFServer got oid:"+oid);

		 // insertProxy(serverName);

		 addModule(proxy, oid);

		return proxy;
	}	

/**
 * It is called internally.
 */
      public void insertRemoteServer(String serverName,String objectID,
				     org.omg.CORBA.Object serverContext) 
      {

	System.out.println("insertRemoteServer is CALLED with:"+serverName+
			   " cont:"+serverContext+" ID:"+objectID);

	waitUntilSlaveReady(serverContext);

	org.omg.CORBA.Object proxy=null;
	proxy   = factory.prevChild().createProxy(serverName, serverContext);

	//set proxy for remote Slave server
	BeanContextChild bcc = BeanContextChildHelper.narrow(serverContext);
	bcc.setMyProxy(proxy);

	System.out.println("insertRemoteServer before addModule");
	//Chech myID is prefix of objectID:eg myID/xxxx
	
	if (myPrefixEqual(objectID))
	  addModule(proxy, objectID);
	else{
	  WebFlowContext wfc=WebFlowContextHelper.narrow(getParentContext(objectID));
	  wfc.addModule(proxy, objectID);
	  bcc.setObjectID(objectID);
	}
	System.out.println("insertRemoteServer After addModule");
      }	

public boolean  myPrefixEqual(String otherID)
{
  String myID = getObjectID();
  int indx_ =  otherID.lastIndexOf("/");
  if (indx_ <0)
    indx_ = otherID.length();
  String prefix = otherID.substring(0,indx_);
  System.out.println("insertRemoteServer:prefix:"+prefix); 
  if (prefix.equals(myID))
    return true;
  else
    return false;
}


public void waitUntilSlaveReady( org.omg.CORBA.Object obj)
{
  boolean flag=true;
  while(flag){
    System.out.println("Waiting for slave to be ready...");
    boolean non_exist = true;
    try{
      non_exist = obj._non_existent();
    }catch(org.omg.CORBA.COMM_FAILURE ex){}
    catch(org.omg.CORBA.OBJECT_NOT_EXIST ex){}
    
    if (!non_exist)
      flag = false;
    try{
      Thread.sleep(500);
    }catch(InterruptedException ex){
    }

  }

}

/** 
 * It is used internally
 */
 public org.omg.CORBA.Object insertProxy(String serverName)
	{
		org.omg.CORBA.Object server = configSettings_.getWFServer(serverName);
		System.out.println("getWFServer :server:"+server);
		org.omg.CORBA.Object proxy=null;
		  
		WebFlowContext wc = WebFlowContextHelper.narrow(server);
		String bcname   = "proxy_user"+serverName;
		proxy   = factory.prevChild().createProxy(bcname, server);
		System.out.println("getWFServer :proxy:"+proxy);
		
		//set my Proxy
		//If server accessed is local
		if (serverName.equals(ContextName))
		    setMyProxy(proxy);
		//If it is remote
		else 
		    wc.setMyProxy(proxy);

		//write the Proxy of the server into url
		boolean non_exist = configSettings_.writeGatewayIOR(proxy);
		return proxy;
	}	
	

  public WebFlowContext  getWebFlowContext(org.omg.CORBA.Object product)
  {
    System.out.println("getWebFlowContext received :"+product);
    /*Proxy p = ProxyHelper.narrow(product);
    System.out.println("getWebFlowContext got proxy:"+p);
    
    WebFlowContext ws = WebFlowContextHelper.narrow(p.getWebFlowContext());
    System.out.println("getWebFlowContext got wc:"+ws);
    */

    BeanContextChild bcc = BeanContextChildHelper.narrow(product);
    WebFlowContext   wfc = WebFlowContextHelper.narrow(bcc.getBeanContext());
    return wfc;
  }

  public String[] getModuleList()
  {
    return  configSettings_.getModuleList();
  }
  	
  public moduleFactory getMFImpl()
  {
	return MFImpl;
  }

  //this method will be called by ModuleFactory Server
  public void start(org.omg.CORBA.Object factoryObj)
  {
    System.out.println("start method of WebFlowContext is called:"+ContextName);
    MFImpl = moduleFactoryHelper.narrow(factoryObj);
   }

public void setClorAndMFI(int myColor, moduleFactory MFImpl)
  {
    this.myColor = myColor;
    this.MFImpl  = MFImpl;
    this.factory = MFImpl;
    System.out.println("setClorAndMFI:Context "+ContextName+" 's Color :"+myColor);
    
  }

} //end of WebFlowContextOps Class


